@@ -12,6 +12,7 @@ from shapely.geometry import Polygon, MultiPolygon
 from shapely.wkt import dumps
 
 from pyiem import reference
+from pyiem.util import LOG
 from pyiem.exceptions import TextProductException
 from pyiem.nws import ugc, vtec, hvtec
 
@@ -20,12 +21,19 @@ from pyiem.nws import ugc, vtec, hvtec
 # but in practice it is sometimes something between 4 and 6 chars
 # We do require that the first character be a A-Z one as otherwise this will
 # match the LDM sequence number at the top!
-AFOSRE = re.compile(r"^([A-Z][A-Z0-9\s]{3,5})$", re.M)
-TIME_RE = re.compile(
-    "^([0-9:]+) (AM|PM) ([A-Z][A-Z][A-Z]?T) ([A-Z][A-Z][A-Z]) "
-    "([A-Z][A-Z][A-Z]) ([0-9]+) ([1-2][0-9][0-9][0-9])$",
-    re.M | re.IGNORECASE,
+AFOSRE = re.compile(r"^([A-Z][A-Z0-9]{3,5})\s*\t*$", re.M)
+TIME_FMT = (
+    "([0-9:]+) (AM|PM) ([A-Z][A-Z][A-Z]?T) ([A-Z][A-Z][A-Z]) "
+    "([A-Z][A-Z][A-Z]) ([0-9]+) ([1-2][0-9][0-9][0-9])"
 )
+TIME_RE = re.compile(f"^{TIME_FMT}$", re.M | re.IGNORECASE)
+# Sometimes products have a duplicated timestamp in another tz
+TIME_EXT_RE = re.compile(
+    rf"^{TIME_FMT}\s?/\s?{TIME_FMT}\s?/$", re.M | re.IGNORECASE
+)
+# Without the line start and end requirement
+TIME_RE_ANYWHERE = re.compile(f"{TIME_FMT}", re.IGNORECASE)
+
 # Note that bbb of RTD is supported here, but does not appear to be allowed
 WMO_RE = re.compile(
     "^(?P<ttaaii>[A-Z0-9]{4,6}) (?P<cccc>[A-Z]{4}) "
@@ -157,7 +165,7 @@ def date_tokens2datetime(tokens):
     )
     # Careful here, need to go to UTC time first then come back!
     now = datetime.strptime(dstr, "%I:%M %p %b %d %Y")
-    now += timedelta(hours=reference.offsets[z])
+    now += timedelta(hours=reference.offsets.get(z, 0))
     return z, tz, now.replace(tzinfo=timezone.utc)
 
 
@@ -213,6 +221,24 @@ class TextProductSegment:
             return None
         return self.hvtec[0].nwsli.id
 
+    def get_hvtec_cause(self):
+        """ Return the first hvtec cause entry, if it exists """
+        if not self.hvtec:
+            return None
+        return self.hvtec[0].cause
+
+    def get_hvtec_severity(self):
+        """ Return the first hvtec severity entry, if it exists """
+        if not self.hvtec:
+            return None
+        return self.hvtec[0].severity
+
+    def get_hvtec_record(self):
+        """ Return the first hvtec record entry, if it exists """
+        if not self.hvtec:
+            return None
+        return self.hvtec[0].record
+
     def svs_search(self):
         """ Special search the product for special text """
         sections = self.unixtext.split("\n\n")
@@ -287,13 +313,15 @@ class TextProductSegment:
         """
         Convert the special tags into a nice text
         """
-        if (
-            self.windtag is None
-            and self.tornadotag is None
-            and self.hailtag is None
-            and self.tornadodamagetag is None
-            and self.waterspouttag is None
-            and not self.flood_tags
+        if all(
+            [
+                self.windtag is None,
+                self.tornadotag is None,
+                self.hailtag is None,
+                self.tornadodamagetag is None,
+                self.waterspouttag is None,
+                not self.flood_tags,
+            ]
         ):
             return ""
 
@@ -354,13 +382,13 @@ class TextProductSegment:
             self.tp.warnings.append(
                 ("LAT...LON polygon is invalid!\n%s") % (poly.exterior.xy,)
             )
-            return
+            return None
         # check 2, is the exterior ring of the polygon clockwise?
         if poly.exterior.is_ccw:
             # No longer a warning as it was too much noise
-            print(
-                ("LAT...LON polygon exterior is CCW, reversing\n%s")
-                % (poly.exterior.xy,)
+            LOG.info(
+                "LAT...LON polygon exterior is CCW, reversing\n%s",
+                poly.exterior.xy,
             )
             poly = Polygon(
                 zip(poly.exterior.xy[0][::-1], poly.exterior.xy[1][::-1])
@@ -431,8 +459,8 @@ class TextProductSegment:
         return headlines
 
     def get_affected_wfos(self):
-        """ Based on the ugc_provider, figure out which WFOs are impacted by
-        this product segment """
+        """Based on the ugc_provider, figure out which WFOs are impacted by
+        this product segment"""
         affected_wfos = []
         for _ugc in self.ugcs:
             for wfo in _ugc.wfos:
@@ -496,6 +524,16 @@ class TextProduct:
         if parse_segments:
             self.parse_segments()
 
+    def suv_iter(self):
+        """Yield [(segment, ugcs, vtec)] combos found in product."""
+        for segment in self.segments:
+            if not segment.ugcs or not segment.vtec:
+                continue
+            for _vtec in segment.vtec:
+                if _vtec.status == "T" or _vtec.action == "ROU":
+                    continue
+                yield (segment, segment.ugcs, _vtec)
+
     def is_resent(self):
         """ Check to see if this product is a ...RESENT product """
         return self.unixtext.find("...RESENT") > 0
@@ -523,7 +561,10 @@ class TextProduct:
 
     def get_channels(self):
         """ Return a list of channels """
-        return [self.afos, f"{self.afos[:3]}..."]
+        res = [self.afos, f"{self.afos[:3]}..."]
+        if self.afos[:3] in ["TCU", "TCD", "TCM", "TCP", "TWO"]:
+            res.append(self.afos[:5])
+        return res
 
     def get_nicedate(self):
         """Nicely format the issuance time of this product"""
@@ -590,8 +631,7 @@ class TextProduct:
         return res
 
     def get_signature(self):
-        """ Find the signature at the bottom of the page
-        """
+        """Find the signature at the bottom of the page"""
         return " ".join(
             self.segments[-1].unixtext.replace("\n", " ").strip().split()
         )
@@ -613,7 +653,7 @@ class TextProduct:
         return pid.strip()
 
     def _parse_valid(self, provided_utcnow):
-        """ Figure out the timestamp of this product.
+        """Figure out the timestamp of this product.
 
         Args:
           provided_utcnow (datetime): What our library was provided for the UTC
@@ -622,9 +662,15 @@ class TextProduct:
         # The MND header hopefully has a full timestamp that is the best
         # truth that we can have for this product.
         tokens = TIME_RE.findall(self.unixtext)
+        if not tokens:
+            tokens = TIME_EXT_RE.findall(self.unixtext)
+            if not tokens:
+                tokens = TIME_RE_ANYWHERE.findall(self.unixtext)
         if provided_utcnow is None and tokens:
             try:
-                _z, _tz, valid = date_tokens2datetime(tokens[0])
+                z, _tz, valid = date_tokens2datetime(tokens[0])
+                if z not in reference.offsets:
+                    self.warnings.append(f"product timezone '{z}' unknown")
             except ValueError:
                 msg = (
                     "Invalid timestamp [%s] found in product "
@@ -690,8 +736,8 @@ class TextProduct:
             self.wmo += "00"
 
     def get_affected_wfos(self):
-        """ Based on the ugc_provider, figure out which WFOs are impacted by
-        this product """
+        """Based on the ugc_provider, figure out which WFOs are impacted by
+        this product"""
         affected_wfos = []
         for segment in self.segments:
             for ugcs in segment.ugcs:
@@ -707,6 +753,6 @@ class TextProduct:
         data = "\n".join(
             [line.strip() for line in self.sections[0].split("\n")[:4]]
         )
-        tokens = re.findall("^([A-Z0-9 ]{4,6})$", data, re.M)
+        tokens = re.findall(r"^([A-Z0-9\s\t]{4,6})$", data, re.M)
         if tokens:
-            self.afos = tokens[0]
+            self.afos = tokens[0].strip()
