@@ -25,7 +25,6 @@ import subprocess
 import shutil
 import pickle
 import datetime
-import math
 import warnings
 
 #
@@ -72,8 +71,9 @@ from pyiem.reference import (  # noqa: F401  # pylint: disable=unused-import
     Z_POLITICAL,
     Z_OVERLAY,
     Z_OVERLAY2,
+    Z_FRAME,
 )
-from pyiem.util import ssw, logger
+from pyiem.util import ssw, LOG
 from pyiem.datatypes import speed, direction
 from pyiem.plot.colormaps import stretch_cmap
 import pyiem.meteorology as meteorology
@@ -82,7 +82,6 @@ import pyiem.meteorology as meteorology
 cartopy.config["pre_existing_data_dir"] = "/opt/miniconda3/cartopy_data/"
 # Set a saner default for apache et al
 cartopy.config["data_dir"] = "/tmp/"
-LOG = logger()
 
 
 DATADIR = os.sep.join([os.path.dirname(__file__), "..", "data"])
@@ -90,24 +89,6 @@ MAIN_AX_BOUNDS = [0.01, 0.05, 0.898, 0.85]
 CAX_BOUNDS = [0.917, 0.1, 0.02, 0.8]
 
 
-def centered_bins(absmax, _on=0, bins=9):
-    """Return a smooth binning
-
-    Args:
-      absmax (real): positive absolute value we want or bins to enclose
-      on (real): where to center the bins at (TODO)
-      bins (int): number of bins to generate
-    Returns:
-      list of bins"""
-    mx = (bins - 1) / 2.0
-    width = absmax / float(mx)
-    if width > 1:
-        width = math.ceil(width)
-    else:
-        return np.linspace(0 - absmax, absmax, bins)
-    return np.arange(0 - mx, mx + 1) * width
-
-
 def true_filter(_bm, _key, _val):
     """Always return true"""
     return True
@@ -151,7 +132,7 @@ def load_pickle_pd(filename):
     fn = "%s/%s" % (DATADIR, filename)
     if not os.path.isfile(fn):
         LOG.info("load_pickle_pd(%s) failed, file is missing", fn)
-        return fn
+        return
     return pd.read_pickle(fn)
 
 
@@ -229,6 +210,9 @@ class MapPlot:
         self.cax = plt.axes(CAX_BOUNDS, frameon=False, yticks=[], xticks=[])
         self.axes = []
         self.ax = None
+        self.pr_ax = None
+        self.hi_ax = None
+        self.ak_ax = None
         # hack around sector=iowa
         if self.sector == "iowa":
             self.sector = "state"
@@ -245,9 +229,6 @@ class MapPlot:
             _a.add_feature(cfeature.LAND, facecolor=_c, zorder=Z_CF)
             _a.add_feature(cfeature.COASTLINE, lw=1.0, zorder=Z_POLITICAL)
             _a.add_feature(cfeature.BORDERS, lw=1.0, zorder=Z_POLITICAL)
-            _a.add_feature(
-                cfeature.OCEAN, facecolor=(0.4471, 0.6235, 0.8117), zorder=Z_CF
-            )
             _a.add_feature(
                 cfeature.LAKES, facecolor=(0.4471, 0.6235, 0.8117), zorder=Z_CF
             )
@@ -691,7 +672,8 @@ class MapPlot:
           labeltextsize (int, optional): Size of the label text
           labelcolor (str, optional): Color to use for drawing labels
           showmarker (bool, optional): Place a marker on the map for the label
-          labelbuffer (int): pixel buffer around labels
+          labelbuffer (int): pixel buffer around labels, a value of `0`
+            disables the label culling logic.
           outlinecolor (color): color to use for text outlines
           zorder (int or list, optional): zorder to use for plotting.
           textoutlinewidth (int): width of the font outline, default 3.
@@ -717,7 +699,8 @@ class MapPlot:
         figwidth = bbox.width * self.fig.dpi
         figheight = bbox.height * self.fig.dpi
         if self.textmask is None:
-            self.textmask = np.zeros((int(figwidth), int(figheight)), bool)
+            # zorder is used to track plotting priorities
+            self.textmask = np.zeros((int(figwidth), int(figheight)), np.int8)
         thisax = self.ax
         # Create a fake label, to test out our scaling
         t0 = self.fig.text(
@@ -772,17 +755,13 @@ class MapPlot:
                     (imgy0 + mystr_lines * ypixels + 2 * labelbuffer * 0.75),
                 ]
             )
-            _cnt = np.sum(
-                np.where(
-                    self.textmask[
-                        int(imgx0) : int(imgx1), int(imgy0) : int(imgy1)
-                    ],
-                    1,
-                    0,
-                )
+            overlap = (
+                self.textmask[int(imgx0) : int(imgx1), int(imgy0) : int(imgy1)]
+                >= z
             )
+            _cnt = np.sum(overlap)
             # If we have more than 15 pixels of overlap, don't plot this!
-            if _cnt > 15:
+            if _cnt > 15 and labelbuffer > 0:
                 if self.debug:
                     LOG.info(
                         "culling |%s| due to overlap, %s", repr(mystr), _cnt
@@ -819,7 +798,14 @@ class MapPlot:
                 )
             self.textmask[
                 int(imgx0) : int(imgx1), int(imgy0) : int(imgy1)
-            ] = True
+            ] = np.where(
+                self.textmask[int(imgx0) : int(imgx1), int(imgy0) : int(imgy1)]
+                < z,
+                z,
+                self.textmask[
+                    int(imgx0) : int(imgx1), int(imgy0) : int(imgy1)
+                ],
+            )
             t0 = thisax.text(
                 o,
                 a,
@@ -925,11 +911,28 @@ class MapPlot:
         return _hex
 
     def pcolormesh(self, lons, lats, vals, clevs, **kwargs):
-        """ pcolormesh wrapper """
+        """Opinionated mpl.pcolormesh wrapper.
+
+        If you supply a lons in the same shape of the vals, this method will
+        tack on an extra row and column to make matplotlib happy. If you do
+        not want this, then pass your own lons + lats that is 1 column and 1
+        row greater than vals.
+        """
         cmap = stretch_cmap(
             kwargs.get("cmap"), clevs, extend=kwargs.get("extend")
         )
         norm = mpcolors.BoundaryNorm(clevs, cmap.N)
+        if lons.shape == vals.shape:
+            # Tack on some extra
+            new = lons[-1, :] * 2 - lons[-2, :]
+            lons = np.r_[lons, [new]]
+            new = lats[-1, :] * 2 - lats[-2, :]
+            lats = np.r_[lats, [new]]
+            new = lons[:, -1] * 2 - lons[:, -2]
+            lons = np.c_[lons, new]
+            new = lats[:, -1] * 2 - lats[:, -2]
+            lats = np.c_[lats, new]
+
         res = self.ax.pcolormesh(
             lons,
             lats,
@@ -989,7 +992,7 @@ class MapPlot:
         mask_outside_polygon(points[:, :2], ax=self.ax)
 
     def contourf(self, lons, lats, vals, clevs, **kwargs):
-        """ Contourf
+        """Contourf
 
         Args:
           ilabel (boolean,optional): Should we label contours
@@ -1151,15 +1154,23 @@ class MapPlot:
             for polyi, polygon in enumerate(ugcdict.get("geom", [])):
                 if polygon.exterior is None:
                     continue
-                arr = np.asarray(polygon.exterior)
-                points = self.ax.projection.transform_points(
-                    ccrs.Geodetic(), arr[:, 0], arr[:, 1]
-                )
-                p = Polygon(points[:, :2], fc=c, ec="k", zorder=z, lw=0.1)
-                if z == Z_OVERLAY:
-                    self.ax.add_patch(p)
-                if z == Z_OVERLAY2:
-                    self.ax.add_patch(p)
+                if z in [Z_OVERLAY, Z_OVERLAY2]:
+                    arr = np.asarray(polygon.exterior)
+                    # Figure out which ax this goes with
+                    ax = self.ax
+                    if self.sector == "nws":
+                        if ugc.startswith("PR"):
+                            ax = self.pr_ax
+                        elif ugc.startswith("AK"):
+                            ax = self.ak_ax
+                        elif ugc.startswith("HI"):
+                            ax = self.hi_ax
+                    points = ax.projection.transform_points(
+                        ccrs.Geodetic(), arr[:, 0], arr[:, 1]
+                    )
+                    ax.add_patch(
+                        Polygon(points[:, :2], fc=c, ec="k", zorder=z, lw=0.1)
+                    )
                 if polyi == 0 and ilabel:
                     # prefer our stored centroid vs calculated one
                     mx = ugcdict.get("lon", polygon.centroid.x)
@@ -1274,7 +1285,7 @@ class MapPlot:
         )
 
     def drawcounties(self, color="k"):
-        """ Draw counties onto the map
+        """Draw counties onto the map
 
         Args:
           color (color,optional): line color to use
@@ -1343,7 +1354,10 @@ class MapPlot:
 
 def windrose(*args, **kwargs):
     """Depreciated."""
-    warnings.warn("windrose() is depreciated, use pyiem.windrose_utils!")
+    warnings.warn(
+        "windrose() is depreciated, use pyiem.windrose_utils!",
+        DeprecationWarning,
+    )
     import pyiem.windrose_utils as wru
 
     return wru.windrose(*args, **kwargs)
