@@ -11,6 +11,8 @@ from pyiem.nws.nwsli import NWSLI
 from pyiem.nws.ugc import UGCParseException, UGC
 from pyiem.nws.vtec import parse
 
+CUGC = "Product failed to cover all UGC"
+
 
 class FakeObject:
     """Mocked thing"""
@@ -20,11 +22,13 @@ class FakeObject:
     vtec = None
 
 
-@pytest.fixture
+@pytest.fixture(scope="function")
 def dbcursor():
     """Return a disposable database cursor."""
     pgconn = get_dbconn("postgis")
-    return pgconn.cursor(cursor_factory=psycopg2.extras.DictCursor)
+    cursor = pgconn.cursor(cursor_factory=psycopg2.extras.DictCursor)
+    yield cursor
+    pgconn.close()
 
 
 def filter_warnings(ar, startswith="get_gid"):
@@ -36,6 +40,113 @@ def filter_warnings(ar, startswith="get_gid"):
     return [a for a in ar if not a.startswith(startswith)]
 
 
+def test_201120_tml_line(dbcursor):
+    """Test that we can insert TML lines."""
+    data = get_test_file("TORFSD.txt")
+    prod = vtecparser(data.replace("4260 9567", "4260 9567 4160 8567"))
+    prod.sql(dbcursor)
+
+
+def test_201120_sbw_duplicate(dbcursor):
+    """Test that we get an error for a SBW duplicated."""
+    data = get_test_file("TOR.txt")
+    prod = vtecparser(data)
+    prod.sql(dbcursor)
+    prod.sql(dbcursor)
+    assert any([x.find("is a SBW duplicate") > -1 for x in prod.warnings])
+    prod = vtecparser(data.replace(".NEW.", ".CON."))
+    prod.sql(dbcursor)
+    assert any([x.find("SBW prev polygon") > -1 for x in prod.warnings])
+
+
+def test_201120_correction(dbcursor):
+    """Test that we emit a warning for first found correction."""
+    data = get_test_file("TOR.txt").replace("291656", "291656 CCA")
+    prod = vtecparser(data)
+    prod.sql(dbcursor)
+    assert any([x.find("is a correction") > -1 for x in prod.warnings])
+
+
+def test_201120_unknown_vtec(dbcursor):
+    """Test that we emit a warning when finding an unknown VTEC action."""
+    data = get_test_file("TOR.txt").replace(".NEW.KJAN", ".QQQ.KJAN")
+    prod = vtecparser(data)
+    prod.sql(dbcursor)
+    assert any([x.find("QQQ") > -1 for x in prod.warnings])
+
+
+def test_201120_resent(dbcursor):
+    """Test that we can handle a resent product."""
+    data = get_test_file("TOR.txt")
+    prod = vtecparser(data)
+    prod.sql(dbcursor)
+    eas = "EAS ACTIVATION REQUESTED"
+    prod = vtecparser(data.replace(eas, f"{eas}...RESENT"))
+    prod.sql(dbcursor)
+
+
+def test_201120_missing_ugc(dbcursor):
+    """Test that a warning for missing ugc is emitted."""
+    data = get_test_file("CFW/CFWSJU.txt")
+    data = data.replace("PRZ001-002-005-008-211645-", "")
+    prod = vtecparser(data)
+    prod.sql(dbcursor)
+    ans = "UGC is missing for segment that has VTEC!"
+    assert ans in prod.warnings
+
+
+def test_201120_dup(dbcursor):
+    """Test that a warning for duplicated VTEC is emitted."""
+    prod = vtecparser(get_test_file("CFW/CFWSJU.txt"))
+    prod.sql(dbcursor)
+    ans = "Segment has duplicated VTEC"
+    assert any([x.startswith(ans) for x in prod.warnings])
+
+
+def test_201116_1970vtec():
+    """Test that we don't allow a 1970s VTEC timestamp, which is in error."""
+    prod = vtecparser(get_test_file("FLS/FLSSEW_vtec1970.txt"))
+    assert prod.segments[0].vtec[0].begints is None
+
+
+def test_201006_invalid_warning(dbcursor):
+    """Test that we don't complain about a dangling CON statement."""
+    prod = vtecparser(get_test_file("MWWPQR/MWWPQR_0.txt"))
+    prod.sql(dbcursor)
+    assert not filter_warnings(prod.warnings)
+    prod = vtecparser(get_test_file("MWWPQR/MWWPQR_1.txt"))
+    prod.sql(dbcursor)
+    assert not filter_warnings(prod.warnings)
+
+
+def test_issue284_incomplete_update(dbcursor):
+    """Test that we emit warnings when a product fails to update everything."""
+    prod = vtecparser(get_test_file("FFW/FFWLCH_0.txt"))
+    prod.sql(dbcursor)
+    assert not filter_warnings(prod.warnings)
+    prod = vtecparser(get_test_file("FFW/FFSLCH_1.txt"))
+    prod.sql(dbcursor)
+    warnings = filter_warnings(prod.warnings)
+    assert len(warnings) == 1
+    assert warnings[0].startswith(CUGC)
+
+
+def test_201116_timezone():
+    """Test what happens with some timezone fun."""
+    valid = utc(2020, 9, 9, 0, 29)
+    prod = vtecparser(get_test_file("SVR/SVRPSR.txt"), utcnow=valid)
+    ans = "valid at 5:29 PM MST"
+    assert prod.segments[0].vtec[0].get_begin_string(prod) == ans
+
+
+def test_vtec_begin_time_format():
+    """Test that we can call vtec begin_time."""
+    valid = utc(2010, 4, 5, 21, 47)
+    prod = vtecparser(get_test_file("SVRLMK.txt"), utcnow=valid)
+    ans = "valid at 5:47 PM EDT"
+    assert prod.segments[0].vtec[0].get_begin_string(prod) == ans
+
+
 def test_old_windhail_tag():
     """Test that we can parse legacy wind...hail tags."""
     valid = utc(2010, 4, 5, 21, 47)
@@ -211,9 +322,10 @@ def test_TORE_series(dbcursor):
 def test_190102_exb_newyear(dbcursor):
     """See that we properly can find a complex EXB added in new year."""
     for i in range(4):
+        print(f"processing {i}")
         prod = vtecparser(get_test_file("WSWAFG/%s.txt" % (i,)))
         prod.sql(dbcursor)
-        assert not filter_warnings(prod.warnings)
+        assert not filter_warnings(filter_warnings(prod.warnings), CUGC)
     dbcursor.execute(
         """
         SELECT count(*) from warnings_2018 where wfo = 'AFG' and eventid = 127
@@ -230,12 +342,14 @@ def test_181228_issue76_sbwtable(dbcursor):
     prod = vtecparser(get_test_file("FLWMOB/FLS.txt"))
     prod.sql(dbcursor)
     dbcursor.execute(
-        """
-        SELECT hvtec_nwsli from sbw_2018 where wfo = 'MOB' and eventid = 57
-        and phenomena = 'FL' and significance = 'W'
-    """
+        "SELECT * from sbw_2018 where wfo = 'MOB' and eventid = 57 "
+        "and phenomena = 'FL' and significance = 'W' and status = 'NEW'"
     )
-    assert dbcursor.fetchone()["hvtec_nwsli"] == "MRRM6"
+    row = dbcursor.fetchone()
+    assert row["hvtec_nwsli"] == "MRRM6"
+    assert row["hvtec_severity"] == "1"
+    assert row["hvtec_cause"] == "ER"
+    assert row["hvtec_record"] == "NO"
     assert not filter_warnings(prod.warnings)
 
 
@@ -285,13 +399,8 @@ def test_180202_issue54(dbcursor):
     def get_expire(colname):
         """get expiration"""
         dbcursor.execute(
-            """
-        SELECT distinct """
-            + colname
-            + """ from warnings_2018
-        WHERE wfo = 'LWX' and eventid = 6 and phenomena = 'WW'
-        and significance = 'Y'
-        """
+            f"SELECT distinct {colname} from warnings_2018 WHERE wfo = 'LWX' "
+            "and eventid = 6 and phenomena = 'WW' and significance = 'Y'"
         )
         assert dbcursor.rowcount == 1
         return dbcursor.fetchone()[0]
@@ -531,7 +640,7 @@ def test_151225_extfuture(dbcursor):
     # /O.EXT.KPAH.FL.W.0093.151227T0358Z-151229T0442Z/
     prod = vtecparser(get_test_file("FLWPAH/FLWPAH_2.txt"))
     prod.sql(dbcursor)
-    warnings = filter_warnings(prod.warnings)
+    warnings = filter_warnings(filter_warnings(prod.warnings), CUGC)
     assert len(warnings) == 1
 
 
@@ -763,8 +872,7 @@ def test_141212_mqt(dbcursor):
         print("Parsing Product: %s.txt" % (i,))
         prod = vtecparser(get_test_file("MWWMQT/%i.txt" % (i,)))
         prod.sql(dbcursor)
-        warnings = filter_warnings(prod.warnings)
-        assert not warnings
+        assert not filter_warnings(filter_warnings(prod.warnings), CUGC)
 
 
 def test_141211_null_expire(dbcursor):
@@ -789,18 +897,17 @@ def test_141210_continues(dbcursor):
 def test_141208_upgrade(dbcursor):
     """ See that we can handle the EXB case """
     for i in range(0, 18):
-        print("Processing %s" % (i,))
+        print(f"Processing {i}")
         prod = vtecparser(get_test_file("MWWLWX/%02i.txt" % (i,)))
         prod.sql(dbcursor)
         warnings = filter_warnings(prod.warnings)
         warnings = filter_warnings(warnings, "Segment has duplicated")
-        assert not warnings
+        assert not filter_warnings(warnings, CUGC)
     # ANZ532 gets too entries from the above check the issuance time of first
     dbcursor.execute(
-        """SELECT issue at time zone 'UTC' from warnings_2014
-    where wfo = 'LWX' and eventid = 221
-    and phenomena = 'SC' and significance = 'Y'
-    and ugc = 'ANZ532' and status != 'UPG'"""
+        "SELECT issue at time zone 'UTC' from warnings_2014 where wfo = 'LWX' "
+        "and eventid = 221 and phenomena = 'SC' and significance = 'Y' "
+        "and ugc = 'ANZ532' and status != 'UPG'"
     )
     assert dbcursor.fetchone()[0] == datetime.datetime(2014, 12, 7, 19, 13)
 
