@@ -13,6 +13,27 @@ from pyiem.nws.products import parser as productparser
 from pyiem.util import utc, get_test_file
 
 
+def test_tropical_channels():
+    """Test the channels we have some products go into."""
+    prod = productparser(get_test_file("tropical/TCUCP1.txt"))
+    j = prod.get_jabbers("")
+    assert "TCUCP" in j[0][2]["channels"].split(",")
+
+
+def test_first_flapping():
+    """Test for product crossing month backwards, prevent flapping test."""
+    # Scenario is utcnow is the first and the WMO header is > 25th
+    utcnow = utc(2012, 6, 1)
+    prod = productparser(get_test_file("SVRBMX.txt"), utcnow=utcnow)
+    assert prod.valid == utc(2012, 5, 31, 23, 11)
+
+
+def test_no_afos():
+    """Test product without AFOS/AWIPS ID."""
+    with pytest.raises(TextProductException):
+        productparser(get_test_file("PMDSA.txt").replace("PMDSA ", ""))
+
+
 def test_str2polygon():
     """Test our str2polygon implementation."""
     res = str2polygon("4400 3200 4500 3300 4400 3300")
@@ -56,6 +77,39 @@ def test_wpc():
     assert res[0][0] == ans
 
 
+def test_200913_dualtime():
+    """Process a HLS in two timezones, sigh."""
+    tp = productparser(get_test_file("HLS.txt"))
+    assert not tp.warnings
+    assert tp.z == "EDT"
+
+
+def test_200731_cvt():
+    """See that we handle CVT timezone products."""
+    tp = productparser(get_test_file("TCDAT5_CVT.txt"))
+    res = tp.get_jabbers("http://localhost")
+    ans = (
+        "NHC issues TCDAT5 (TCD) at Jul 31, 8:00 PM CVT "
+        "http://localhost?pid=202007312100-KNHC-WTNT45-TCDAT5"
+    )
+    assert res[0][0] == ans
+
+
+def test_200731_bogus_timezone():
+    """Test that we don't bomb out with unknown timezone."""
+    tp = productparser(
+        get_test_file("TCDAT5_CVT.txt").replace(" CVT ", " ZZT ")
+    )
+    res = tp.get_jabbers("http://localhost")
+    ans = (
+        "NHC issues TCDAT5 (TCD) at Jul 31, 8:00 PM ZZT "
+        "http://localhost?pid=202007312000-KNHC-WTNT45-TCDAT5"
+    )
+    assert res[0][0] == ans
+    assert len(tp.warnings) == 1
+    assert tp.warnings[0] == "product timezone 'ZZT' unknown"
+
+
 def test_180321_mst():
     """Do we do the right thing with MST products whilst in DST"""
     tp = productparser(get_test_file("AFDMST.txt"))
